Package: CytoPipeline
Title: Automation and visualization of flow cytometry data analysis
        pipelines
Version: 1.13.0
Authors@R: 
    c(person(given = "Philippe",
             family = "Hauchamps",
             role = c("aut", "cre"),
             email = "philippe.hauchamps@uclouvain.be",
             comment = c(ORCID = "0000-0003-2865-1852")),
      person(given = "Laurent", family = "Gatto",
             email = "laurent.gatto@uclouvain.be",
             role = "aut",
             comment = c(ORCID = "0000-0002-1520-2268")),
      person(given = "Dan",
             family = "Lin",
             role = "ctb",
             email = "dan.8.lin@gsk.com"))
Description: This package provides support for automation and visualization of
 flow cytometry data analysis pipelines. In the current state, the package
 focuses on the preprocessing and quality control part. The framework is based
 on two main S4 classes, i.e. CytoPipeline and CytoProcessingStep. The pipeline
 steps are linked to corresponding R functions - that are either provided in
 the CytoPipeline package itself, or exported from a third party package,
 or coded by the user her/himself. The processing steps need to be specified
 centrally and explicitly using either a json input file or through step by step
 creation of a CytoPipeline object with dedicated methods. After having run the
 pipeline, obtained results at all steps can be retrieved and visualized thanks
 to file caching (the running facility uses a BiocFileCache implementation).
 The package provides also specific visualization tools like
 pipeline workflow summary display, and 1D/2D comparison plots of obtained
 flowFrames at various steps of the pipeline.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
BugReports: https://github.com/UCLouvain-CBIO/CytoPipeline/issues
URL: https://uclouvain-cbio.github.io/CytoPipeline
biocViews: FlowCytometry, Preprocessing, QualityControl, WorkflowStep,
        ImmunoOncology, Software, Visualization
Collate: 'CytoPipeline-functions.R' 'CytoPipeline-package.R'
        'CytoPipelineClass.R' 'CytoProcessingStep.R'
        'CytoProcessingStepImplementations.R' 'data.R' 'gating.R'
        'utils.R' 'ggplots.R'
Depends: R (>= 4.4)
Imports: methods, stats, utils, withr, rlang, ggplot2 (>= 3.4.1),
        ggcyto, BiocFileCache, BiocParallel, flowCore, PeacoQC, flowAI,
        diagram, jsonlite, scales
Suggests: testthat (>= 3.0.0), vdiffr, diffviewer, knitr, rmarkdown,
        BiocStyle, reshape2, dplyr, CytoPipelineGUI
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/CytoPipeline
git_branch: devel
git_last_commit: 723595f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:08:13 UTC; biocbuild
Author: Philippe Hauchamps [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2865-1852>),
  Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>),
  Dan Lin [ctb]
Maintainer: Philippe Hauchamps <philippe.hauchamps@uclouvain.be>
