# Generated by roxygen2: do not edit by hand

S3method(as.list,CytoPipeline)
S3method(as.list,CytoProcessingStep)
export("experimentName<-")
export("pData<-")
export("sampleFiles<-")
export(CytoProcessingStep)
export(addProcessingStep)
export(aggregateAndSample)
export(appendCellID)
export(applyScaleTransforms)
export(areFluoCols)
export(areSignalCols)
export(as.json.CytoProcessingStep)
export(buildCytoPipelineFromCache)
export(checkCytoPipelineConsistencyWithCache)
export(cleanProcessingSteps)
export(collectNbOfRetainedEvents)
export(compensateFromMatrix)
export(computeScatterChannelsLinearScale)
export(deleteCytoPipelineCache)
export(estimateScaleTransforms)
export(execute)
export(executeProcessingStep)
export(experimentName)
export(export2JSONFile)
export(findTimeChannel)
export(from.json.CytoProcessingStep)
export(getAcquiredCompensationMatrix)
export(getCPSARGS)
export(getCPSFUN)
export(getCPSName)
export(getChannelNamesFromMarkers)
export(getCytoPipelineExperimentNames)
export(getCytoPipelineFlowFrame)
export(getCytoPipelineObjectFromCache)
export(getCytoPipelineObjectInfos)
export(getCytoPipelineScaleTransform)
export(getFCSFileName)
export(getNbProcessingSteps)
export(getProcessingStep)
export(getProcessingStepNames)
export(getTransfoParams)
export(ggplotEvents)
export(ggplotFilterEvents)
export(ggplotFlowRate)
export(pData)
export(plotCytoPipelineProcessingQueue)
export(qualityControlFlowAI)
export(qualityControlPeacoQC)
export(readRDSObject)
export(readSampleFiles)
export(removeChannels)
export(removeDeadCellsManualGate)
export(removeDebrisManualGate)
export(removeDoubletsCytoPipeline)
export(removeMarginsPeacoQC)
export(removeProcessingStep)
export(resetCellIDs)
export(runCompensation)
export(sampleDisplayNames)
export(sampleFiles)
export(sampleNameFromDisplayName)
export(showProcessingSteps)
export(singletsGate)
export(subsample)
export(updateMarkerName)
export(writeFlowFrame)
exportClasses(CytoPipeline)
exportClasses(CytoProcessingStep)
exportMethods(CytoPipeline)
exportMethods(show)
import(ggplot2)
importFrom(ggcyto,scale_x_flowjo_biexp)
importFrom(ggcyto,scale_x_logicle)
importFrom(ggcyto,scale_y_flowjo_biexp)
importFrom(ggcyto,scale_y_logicle)
importFrom(rlang,.data)
importFrom(utils,head)
importMethodsFrom(methods,show)
